using namespace std;

#include <iostream.h>
#include <fstream.h>
#include <math.h>
#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <map>
#include <string>

const int MAX_ALIGN_SIZE=15; //maximum number of species
const int MAX_W=20; // biggest motif size
double INITIAL_BRANCH=0.1;
const double MOTIF_PRIOR=0.001;
const double MIN_FREQ=0.001;
const double MIN_RATE=0.001;
const double MAX_RATE=10.0;


//control file parameters
int MAX_ITER=100;
double EXPECTED_NUMBER=1.0;
double MIN_POST_PROB=0.9;
bool PRINT_TO_SCREEN=1;
int UPDATE_MOTIF_RATES=0;
bool UPDATE_FREQ=0;
bool BOTH_STRANDS=0;
int EVOL_MODEL=1;
double RATE_PCOUNT=1.0;
int N_MOTIFS=1;
double EPSILON=0.01;
int PRINT_OUT=1;
int SAME_TOPO=1;
int W=10;
int OVERLAP=W;
bool TURN_OFF_HEURISTICS=0;


struct sequence { // sequence object - 2 copies of the sequence, a string, and as ints
	int length;
	string name;
	string bases;
	int * asints;

	sequence();
	bool integerize();
	bool print(int start, int stop);
};

struct node {
	bool leaf; bool root;
	struct node * anc;
	struct node * left;
	struct node * right;
	double abr,lbr,rbr;
	//double * seq_post;
	int base; int number;

	node();
	bool print();
};

struct tree { //hold a list of nodes, a pointer to the root and a some substitution matrices
	struct node * nodelist;
	int Nnodes;
	struct node * root;
	double **** psubs;

	tree();
	bool read_from_file(string treefile);
	bool print();
	bool compute_PJC();
	bool compute_PF81(double * bgf);
	bool initialize_pairwise(double dis); //puts root in the middle;
	bool compute_F81_branches(struct alignment * ali, int MAX_ITER, double err); //only pairwise, puts root in the middle
	bool compute_average_F81_branches(struct alignment * ali, int na, int MAX_ITER, double err);
	//double LH(double * bgf);
	//double K();
};

struct alignment { // alignment - holds some sequences, positions of the gaps. tree
	int length;
	int Nseqs;
	int gapped;
	int bad;
	int leftend;
	int rightend;
	string name;
	struct sequence * seq;
	int * nextgap;
	int * nextbad;
	struct tree phyl;
	//struct tree global_tree;
	double treelength;

	alignment();
	bool read_fasta(string filename);
	bool print();
	float div();
	//double * postJC(float alpha, int pos, double * priors); // pairwise only
	bool compute_nextgap();

	//double PgivenBG(int start, int stop); // CCD
	//double PgivenM(int start, int w, double ** freq, double * alpha);
	//bool P_BG_COMPUTED;
	//double * pbg;
	double * bgfreq;

	//double logLH(struct motif mod); // log P(alignment|model)

	bool read_tree(string filename, double * bgf);
	//bool optimize_branch_lengths(double err, int MAX_ITER);
	//bool optimize_branches_individ(double err, int MAX_ITER);
	//bool optimize_branch_length(int n, double err, int MAX_ITER);
	//double K(int start, int window);
};


struct motif { // holds a pwm and vector of evol. rates
	int length;
	string name;
	double ** f;
	double pri;
	double * K;
	double br;
	motif * rc;
	//struct tree * phyl;

	motif();
	//bool intialize_with_ML_word(struct alignment * ALI, int na, struct motif * bg);
	bool initialize_neutral_motif(int w, double pseudocounts);
	bool read_from_file(string mfile);
	bool print();
	double halpern_bruno(int pos);
	struct motif make_rc();
	//bool optimize_by_EM( struct SequenceData * sd,bool update_prior, bool update_matrix);
	double Pr(int s[], int start);

};

double p_subtree_given_base (struct alignment * ali, struct node * n, int base, int pos);
double PJC (int a, int b, double alpha);
double PF81 (int a, int b, double alpha, double * bgf);
double PHB (int a, int b, double alpha, double * f, double * bgf);
double div_to_subs_JC (double R);
double p_subtree_given_motif (struct alignment * ali, struct node * n, double * f, double k, int base, int pos);


int main(int argc, char * argv[]) {


	ifstream ctl("evm.ctl");
	if (!ctl) { cerr << "can't find the control file (evm.ctl)" << endl; return 0; }
	string mfile; string startcons; int GIVEN_MOTIF=0; //0 nothing; 1 consensus; 2 matrix
	int pos,posraten=0; double * RATE_MODEL= new double [MAX_W];
	for (pos=0;pos<MAX_W;pos++) RATE_MODEL[pos]=1.0;

	char comm='?';
	while (!ctl.fail()) { ctl>>comm; //parse the control file
		if (comm == '-') { ctl>>comm;
			if (comm == 'i') ctl >> MAX_ITER;
			else if (comm == 'w') {	ctl >> W;  OVERLAP=W; }
			else if (comm == 'p') { if (posraten<MAX_W) {ctl>>RATE_MODEL[posraten]; posraten++; }}
			else if (comm == 'r') ctl >> UPDATE_MOTIF_RATES;
			else if (comm == 'u') ctl >> UPDATE_FREQ;
			else if (comm == 'd') ctl >> PRINT_TO_SCREEN;
			else if (comm == 'c') { GIVEN_MOTIF=1; ctl >> startcons; }
			else if (comm == 'e') ctl >> EXPECTED_NUMBER;
			else if (comm == 't') ctl >> MIN_POST_PROB;
			else if (comm == 'b') ctl >> BOTH_STRANDS;
			else if (comm == 'm') ctl >> EVOL_MODEL;
			else if (comm == 'n') ctl >> N_MOTIFS;
			else if (comm == 'o') ctl >> PRINT_OUT;
			else if (comm == 's') ctl >> SAME_TOPO;
			else if (comm == 'x') { ctl >> mfile; GIVEN_MOTIF=2; }
			else if (comm == 'h') TURN_OFF_HEURISTICS=1;
		}
	}
	string fastafile;
	string treefile;
	struct alignment * ALI; //store the command line alignments
	struct tree global_tree;
	if (SAME_TOPO==1) {
		treefile="all.tree";
		if (!global_tree.read_from_file(treefile)) { cerr << "all.tree not read"; return 0; }
	}
	//else if ((UPDATE_MOTIF_RATES==1)||(UPDATE_MOTIF_RATES==2)) { cerr<<"can't do ML rates by EM if trees are different"<<endl; return0;}

	if (argc<=1) { cerr << "arguments: [fasta alignment 1] ... " <<endl; return 0;}
	int Nalignments=argc-1;
	ALI = new struct alignment [Nalignments];
	int argnum,a,p,s;
	for (argnum=1;argnum<argc;argnum++) {
		fastafile = argv[argnum];
		if (!ALI[argnum-1].read_fasta(fastafile)) { cerr << "error reading "<< fastafile<<endl; return 0; }
		if (ALI[argnum-1].Nseqs>2) {
			if (SAME_TOPO!=0) treefile="all.tree";
			else treefile=ALI[argnum-1].name + ".tree";
			if (!ALI[argnum-1].read_tree(treefile,ALI[argnum-1].bgfreq)) { cerr << "error reading "<<treefile<<endl; return 0; }
		}
		else if (EVOL_MODEL==1) {
			ALI[argnum-1].phyl.compute_F81_branches(&ALI[argnum-1],MAX_ITER,EPSILON);
			ALI[argnum-1].treelength= ALI[argnum-1].phyl.root->lbr + ALI[argnum-1].phyl.root->rbr;
			if (PRINT_TO_SCREEN) cerr<<ALI[argnum-1].name <<" d(F81)= "<<ALI[argnum-1].treelength<<endl;
		}
	}
	struct motif * M = new struct motif [MAX_ITER]; //motif models
	struct motif RC;
	//initialize the motif model
	int b; char B[4]; B[0]='A'; B[1]='C'; B[2]='G'; B[3]='T';
	int revcom[4]; revcom[0]=3; revcom[1]=2; revcom[2]=1; revcom[3]=0;

	if (GIVEN_MOTIF==2) {
		M[0].read_from_file(mfile);
		if (W!=M[0].length) {
			if (PRINT_TO_SCREEN) cout <<"matrix " << M[0].name << " is not length " <<W<<endl;
			W=M[0].length;
			OVERLAP=M[0].length;
		}
	}
	else if (GIVEN_MOTIF==1) {
		if (PRINT_TO_SCREEN) cout << startcons << " found"<<endl;
		//if (PRINT_TO_SCREEN) if (W!=)
		W=startcons.size();
		OVERLAP=W;
		M[0].initialize_neutral_motif(W,1.0);
		for (pos=0;pos<W;pos++) {
			double tot=0.0;
			for (b=0;b<4;b++) {
				if (B[b] == startcons[pos]) M[0].f[pos][b]+=1.0;
				tot+=M[0].f[pos][b];
			}
			for (b=0;b<4;b++) M[0].f[pos][b] /= tot;
		}
	}
	else M[0].initialize_neutral_motif(W,1.0);

	if (UPDATE_MOTIF_RATES==3) { for (pos=0;pos<W;pos++) M[0].K[pos]=1.0; }
	else { for (pos=0;pos<W;pos++) M[0].K[pos]=RATE_MODEL[pos]; }
	double div_to_subs_JC (double R);
	if (SAME_TOPO==2) {
		double k=0.0; double id=0.0; int totalbp=0;
		for (a=0;a<Nalignments;a++) {
			id += (1.0-ALI[a].div())*((double) (ALI[a].length-ALI[a].bad-ALI[a].gapped));
			totalbp+=(ALI[a].length-ALI[a].bad-ALI[a].gapped);
		}
		k = div_to_subs_JC(((double) totalbp)/id - 1.0);
		global_tree.initialize_pairwise(k);
		for (a=0;a<Nalignments;a++) {
			ALI[a].phyl.initialize_pairwise(k);
			ALI[a].treelength=k;
		}
		if (PRINT_TO_SCREEN) cout << "rate: "<<id<<" "<<totalbp<<" -> "<<k<<endl;
		if (EVOL_MODEL==1) { cerr <<"F81 not implemented. try JC model"<<endl; return 0;}

	}
	else if ((UPDATE_MOTIF_RATES==1)||(UPDATE_MOTIF_RATES==2)) { cout << "ML rates not implemented..."<<endl; return 0; }


	if (EVOL_MODEL==0) for (a=0;a<Nalignments;a++) ALI[a].phyl.compute_PJC();
	if (EVOL_MODEL==1) for (a=0;a<Nalignments;a++) ALI[a].phyl.compute_PF81(ALI[a].bgfreq);

	//BG.initialize_neutral_motif(W,0.0); //intitialize background model
	double gapp=0.0; double badp=0.0; double bp=0.0;
	double ** bglh = new double * [Nalignments]; //store probabilities
	double **** mcond = new double *** [Nalignments]; //(in case we need not recompute)
	double **** rccond;
	bool ** already_found = new bool * [Nalignments];
	if (BOTH_STRANDS) {
		rccond= new double *** [Nalignments];
	}
		RC=M[0].make_rc();
		M[0].rc=&RC;
	
	if (PRINT_TO_SCREEN) cerr<<"calculating initial conditionals...";
	for (a=0;a<Nalignments;a++) { if (PRINT_TO_SCREEN) cerr<< ".";
		bglh[a]=new double [ALI[a].length];
		mcond[a]=new double ** [ALI[a].length];
		already_found[a]=new bool [ALI[a].length];
		if (BOTH_STRANDS) rccond[a]=new double ** [ALI[a].length];
		for (p=ALI[a].leftend;p<ALI[a].rightend;p++) {
			already_found[a][p]=0;
			if ((ALI[a].nextbad[p]>=(p+W))&&(ALI[a].nextgap[p]>=(p+W))) {
				bp+=1.0;
				mcond[a][p]=new double * [W];
				if (BOTH_STRANDS) rccond[a][p]=new double * [W];
				for (pos=0;pos<W;pos++) {
					mcond[a][p][pos] = new double [4];
					if (BOTH_STRANDS) rccond[a][p][pos] = new double [4];
					for (b=0;b<4;b++) {
						mcond[a][p][pos][b]=p_subtree_given_motif(&ALI[a],ALI[a].phyl.root,M[0].f[pos],M[0].K[pos],b,p+pos);
						if (BOTH_STRANDS)
							rccond[a][p][pos][b]=p_subtree_given_motif(&ALI[a],ALI[a].phyl.root,RC.f[pos],RC.K[pos],b,p+pos);
					}
				}
			}
			if (ALI[a].nextbad[p]==p) badp+=1.0;
			else if (ALI[a].nextgap[p]==p) gapp+=1.0;
			else {
				bglh[a][p]=0.0;
				for (b=0;b<4;b++) bglh[a][p]+=ALI[a].bgfreq[b]*p_subtree_given_base(&ALI[a],ALI[a].phyl.root,b,p);
				//for (b=0;b<4;b++) cout << ALI[a].bgfreq[b]<<" "<< p_subtree_given_base(&ALI[a],ALI[a].phyl.root,b,p);
				//cout<<endl;
			}
		}// return 0;
	}
	//for (p=(BG.length-1);p>=0;p--) for (b=0;b<4;b++) BG.f[p][b]= (BG.f[0][b] / bp);
	if (PRINT_TO_SCREEN) cout <<"done"<<endl<< gapp<<" gapped and " << badp << " bad positions in alignments of "<<bp <<" bp"<<endl;

	M[0].pri= ((double) Nalignments) / bp; //set the prior to be 1 per sequence
	M[0].pri*=EXPECTED_NUMBER;
	double lh_pos(double * f, double * cond);
	double lh_given_motif(struct motif * m, double ** cond);
	double lh_given_bg(int start,int stop,double * lhpos);

//loop over the each motif
int n;

ofstream out("EMnEMscores");
for (n=0;n<N_MOTIFS;n++) {
	if (GIVEN_MOTIF==0) {
		if (PRINT_TO_SCREEN) cout <<"trying starting positions..."<<endl;
		int bestp, besta; double bestlh=-1e100;
		for (a=0;a<Nalignments;a++) for (p=ALI[a].leftend;p<(ALI[a].rightend-W);p++)
				if ((ALI[a].nextbad[p]>=(p+W))&&(ALI[a].nextgap[p]>=(p+W))&&(!already_found[a][p])) {
					for (pos=0;pos<W;pos++) {
						double tot=0.0;
						for (b=0;b<4;b++) {
							M[0].f[pos][b]+=(ALI[a].bgfreq[b]*p_subtree_given_base(&ALI[a],ALI[a].phyl.root,b,p+pos)/bglh[a][p]);
							tot+=M[0].f[pos][b];
						}
						for (b=0;b<4;b++) M[0].f[pos][b] /= tot;
					}
					RC=M[0].make_rc();
					M[0].rc=&RC;
					if ((EVOL_MODEL!=0)||(UPDATE_MOTIF_RATES!=0)) { cout << "initialization for JC model only "<<endl; return 0; }
					double thisloglh=0.0; int q;
					for (b=0;b<Nalignments;b++) for (q=ALI[b].leftend;q<(ALI[b].rightend-W);q++)
						if ((ALI[b].nextbad[q]>=(q+W))&&(ALI[b].nextgap[q]>=(q+W))&&(!already_found[b][q])) {
							double pm=M[0].pri*lh_given_motif(&M[0],mcond[b][q]);
							if (BOTH_STRANDS) pm=0.5*(pm+M[0].pri*lh_given_motif(M[0].rc,rccond[b][q]));
							thisloglh+=log((pm+(1.0-M[0].pri)*lh_given_bg(q,q+W,bglh[b])));
					}
					if (thisloglh>bestlh) { //remember the top word
						besta=a; bestp=p; bestlh=thisloglh;
						if (PRINT_TO_SCREEN) {
							for (s=0;s<ALI[a].Nseqs;s++) ALI[a].seq[s].print(p,p+W);
							cout << " "<<thisloglh<<endl;
						}
					}
					for (pos=0;pos<W;pos++) for (b=0;b<4;b++) M[0].f[pos][b]=0.25;
		}
		if (PRINT_TO_SCREEN) cout << "starting with "<<ALI[besta].name<<" position "<<bestp<<" (lh="<<bestlh<<")"<<endl;
		for (pos=0;pos<W;pos++) { //set the motif to be the top word
			double tot=0.0;
			for (b=0;b<4;b++) {
				M[0].f[pos][b]+=(ALI[besta].bgfreq[b]*p_subtree_given_base(&ALI[besta],ALI[besta].phyl.root,b,bestp+pos)/bglh[besta][bestp]);
				tot+=M[0].f[pos][b];
				//cout <<p_subtree_given_base(&ALI[besta],ALI[besta].phyl.root,b,bestp+pos)<<" ";
			}
			for (b=0;b<4;b++) M[0].f[pos][b] /= tot;
			//cout <<endl;
		}
	}
	//return 0;
	if (PRINT_TO_SCREEN) M[0].print();

	//int nmotif=0;
	if (TURN_OFF_HEURISTICS) OVERLAP=0;
	int bestiteration;
	double * loglh = new double [MAX_ITER];
	int i,stable=0;
	for (i=0;i<(MAX_ITER-1);i++) {
		loglh[i]=0.0;
		double En=0.0;
  		M[i+1].pri=M[i].pri;
		M[i+1].length=W;
		M[i+1].K=new double [W];
		M[i+1].f=new double * [W];
		double Ech[W]; double Eid[W]; //used for pairwise
		for (pos=0;pos<W;pos++) {
			if ((UPDATE_MOTIF_RATES==0)||(UPDATE_MOTIF_RATES==3)) M[i+1].K[pos]=M[i].K[pos];
			else if (SAME_TOPO==2) Ech[pos]=Eid[pos]=0.0;
			else { cout << "no ML rates for multiple alignments yet"<<endl; return 0; }
				//else Emrates[pos]=0.0;
			M[i+1].f[pos] = new double [4];
			for (b=0;b<4;b++) M[i+1].f[pos][b]=0.0;
		}
		for (a=0;a<Nalignments;a++) for (p=ALI[a].leftend;p<(ALI[a].rightend-W);p++)
			if ((ALI[a].nextgap[p]>=(p+W))&&(ALI[a].nextbad[p]>=(p+W))&&(!already_found[a][p])) {
				double pbg=(1.0-M[i].pri)*lh_given_bg(p,p+W,bglh[a]);
				double pm=M[i].pri*lh_given_motif(&M[i],mcond[a][p]);
				double prc=0.0;
				if (BOTH_STRANDS) prc=M[i].pri*lh_given_motif(M[i].rc,rccond[a][p]);
				double Em= ((BOTH_STRANDS) ? 0.5*(pm+prc)/(0.5*(pm+prc)+pbg) : pm/(pm+pbg));
				// heuristic to downweight overlaps
				//OVERLAP=0;
				int r= (p>=OVERLAP) ? p-OVERLAP : 0;
				int l= ((p+OVERLAP)<(ALI[a].rightend-W)) ? p+OVERLAP : ALI[a].rightend-W;
				double temptot=0.0;
				for (pos=r;pos<l;pos++) if ((ALI[a].nextgap[pos]>=(pos+W))&&(ALI[a].nextbad[pos]>=(p+W))&&(!already_found[a][pos])) {
					double temp=M[i].pri*lh_given_motif(&M[i],mcond[a][pos]);
					if (BOTH_STRANDS) temp=0.5*(temp+lh_given_motif(M[i].rc,rccond[a][pos]));
					temptot+=(temp/((1.0-M[i].pri)* lh_given_bg(pos,pos+W,bglh[a])+ temp));
				}
				if ((Em>MIN_POST_PROB)&&(PRINT_TO_SCREEN)) { // print out the words that are likely to be motifs
					for (s=0;s<ALI[a].Nseqs;s++) ALI[a].seq[s].print(p,p+W);
					cout <<p<<" p(motif|data)="<<Em<<" "
						<<" ("<<r<<"-"<<l<<":"<<temptot<<") strand:"
						<<0.01*((int) (100.0*pm/(pm+prc)))
						//<<pm/(pm+prc)
						<<" log(LR)="<<log(0.5*(lh_given_motif(&M[i],mcond[a][p])+lh_given_motif(M[i].rc,mcond[a][p])))-log(lh_given_bg(p,p+W,bglh[a]))
					//<<" "<<ALI[a].PgivenM(p,W,M[i-1].f,M[i-1].K)
					//<<" "<<ALI[a].PgivenBG(p,p+W)
					//<<" Phb="<<ALI[an].PgivenM_HB(p,W,M[i-1].f,B[i-1].K)
						<<endl;
				}
				if (temptot>1.0) Em /= temptot;
				En+=Em;
				//if (BOTH_STRANDS) loglh[i]+=log(0.5*(pm+prc) + pbg);
				//else loglh[i]+=log(pm+pbg);
				if (BOTH_STRANDS) loglh[i]+=log(pbg + Em*(0.5*(pm+prc) + pbg));
				else loglh[i]+=log(pbg + Em*(pbg+pm));
				if (UPDATE_FREQ) {
					for (pos=0;pos<W;pos++) {
						double * manc = new double [4];
						double * rcanc = new double [4];
						double motiflh=0.0;
						double rclh=0.0;
						for (b=0;b<4;b++) {
							manc[b]=M[i].f[pos][b]*mcond[a][p][pos][b];
							motiflh+=manc[b];
						}
						for (b=0;b<4;b++) {
							manc[b] /= motiflh;
							M[i+1].f[pos][b] += (manc[b]*Em*(pm/(pm+prc)));
						}
						if (BOTH_STRANDS) {
							for (b=0;b<4;b++) {
								rcanc[b] =RC.f[W-pos-1][b]*rccond[a][p][W-pos-1][b];
								rclh+=rcanc[b];
							}
							for (b=0;b<4;b++) rcanc[b] /= rclh;
							for (b=0;b<4;b++)
								M[i+1].f[pos][b] += (rcanc[revcom[b]]*Em*(prc/(prc+pm)));
						}
					}
				}
				if ((UPDATE_MOTIF_RATES==2)||(UPDATE_MOTIF_RATES==1)) {
					if (SAME_TOPO==2) for (pos=0;pos<W;pos++) {
						double * manc = new double [4];
						double * rcanc = new double [4];
						double motiflh=0.0;
						double rclh=0.0;
						for (b=0;b<4;b++) {
							manc[b]=M[i].f[pos][b]*mcond[a][p][pos][b];
							motiflh+=manc[b];
						}
						if (BOTH_STRANDS) {
							for (b=0;b<4;b++) {
								rcanc[b] =RC.f[pos][b]*rccond[a][p][pos][b];
								rclh+=rcanc[b];
							}
							for (b=0;b<4;b++) rcanc[b] /= rclh;
						}
						for (b=0;b<4;b++) {
							manc[b] /= motiflh;
							double xpb=0.0,ypb=0.0;
							if (ALI[a].seq[0].asints[p+pos]==b) xpb=1.0;
							if (ALI[a].seq[1].asints[p+pos]==b) ypb=1.0;
							if (UPDATE_MOTIF_RATES==2) {
								Ech[pos] += (manc[b]*Em*(2.0-xpb-ypb)*(pm/(pm+prc)));
								Eid[pos] += (manc[b]*Em*(xpb+ypb)*(pm/(pm+prc)));
								if (BOTH_STRANDS) {
									Ech[W-pos-1] += (rcanc[b]*Em*(2.0-xpb-ypb)*(prc/(prc+pm)));
									Eid[W-pos-1] += (rcanc[b]*Em*(xpb+ypb)*(prc/(prc+pm)));
								}
							}
							else {
								Ech[0] += (manc[b]*Em*(2.0-xpb-ypb)*(pm/(pm+prc)));
								Eid[0] += (manc[b]*Em*(xpb+ypb)*(pm/(pm+prc)));
								if (BOTH_STRANDS) {
									Ech[0] += (rcanc[b]*Em*(2.0-xpb-ypb)*(prc/(prc+pm)));
									Eid[0] += (rcanc[b]*Em*(xpb+ypb)*(prc/(prc+pm)));
								}
							}
						}
					}
				}
		}
		for (pos=0;pos<W;pos++) {
			for (b=0;b<4;b++) {
				if (UPDATE_FREQ) {
					M[i+1].f[pos][b] /= En;
					if (M[i+1].f[pos][b]<MIN_FREQ) M[i+1].f[pos][b]=MIN_FREQ;
				}
				else M[i+1].f[pos][b] = M[i].f[pos][b];
			}
		}
		if ((UPDATE_MOTIF_RATES==2)||(UPDATE_MOTIF_RATES==1)) {
			if (SAME_TOPO==2) for (pos=0;pos<W;pos++) {
				double rpos;
				double bgrate=global_tree.root->lbr;
				if (UPDATE_MOTIF_RATES==2) rpos=Ech[pos]/Eid[pos];
				else if (UPDATE_MOTIF_RATES==1) rpos=Ech[0]/Eid[0];
				if (EVOL_MODEL==0) M[i+1].K[pos]=-0.75*log( (1.0-(rpos/3.0)) / (1.0+rpos) )/bgrate;
				if (M[i+1].K[pos]>MAX_RATE) M[i+1].K[pos]=MAX_RATE;
				else if (M[i+1].K[pos]<MIN_RATE) M[i+1].K[pos]=MIN_RATE;
				if ((UPDATE_MOTIF_RATES==2) &&(PRINT_TO_SCREEN)) cout << pos<<": "<<Ech[pos]<< " "<<Eid[pos]<<" -> "<<M[i+1].K[pos]*bgrate<<endl;
				else if ((pos==0)&&(PRINT_TO_SCREEN)) cout << "m: "<<Ech[pos]<< " "<<Eid[pos]<<" -> "<<M[i+1].K[pos]*bgrate<< " ("<<(Ech[pos]+Eid[pos])/((double) W)<<endl;
				//return 0;
			}
		}
		if (i>0) {
			double dlh=loglh[i]-loglh[i-1];
			if (PRINT_TO_SCREEN) {
				if (stable) cout << stable << " ";
				cout <<"motif "<<n<<" ("<<i<<") logLH="<<loglh[i]<<" ("<<dlh<<") "<<En <<" expected"<<endl;
			}
			if (dlh<=EPSILON) stable++;
			else stable=0;
			if (dlh+EPSILON<0.0) cout << "EM warning: likelihood decreased by "<< -dlh<< " at iteration "<< i<<endl;
			else bestiteration=i;
			//else bestiteration=i+1;
			if (stable>15) i=MAX_ITER;
			else {
				RC=M[i+1].make_rc();
				M[i+1].rc=&RC;
				if ((EVOL_MODEL==1)||(UPDATE_MOTIF_RATES!=0)) {
					if (PRINT_TO_SCREEN) cerr<<"recalculating conditionals...";
					for (a=0;a<Nalignments;a++) {
						if (PRINT_TO_SCREEN) cerr<< ".";
						for (p=ALI[a].leftend;p<(ALI[a].rightend);p++)
							if ((ALI[a].nextbad[p]>=(p+W))&&(ALI[a].nextgap[p]>=(p+W)))
								for (pos=0;pos<W;pos++) for (b=0;b<4;b++) {
									mcond[a][p][pos][b]=p_subtree_given_motif(&ALI[a],ALI[a].phyl.root,M[i+1].f[pos],M[i+1].K[pos],b,p+pos);
									if (BOTH_STRANDS) rccond[a][p][pos][b]=p_subtree_given_motif(&ALI[a],ALI[a].phyl.root,RC.f[pos],RC.K[pos],b,p+pos);
								}
					}
					if (PRINT_TO_SCREEN) cerr<<endl;
				}
				if (PRINT_TO_SCREEN) M[i+1].print();
			}
		}
		else {
			RC=M[i+1].make_rc();
   			M[i+1].rc=&RC;
			if ((EVOL_MODEL==1)||(UPDATE_MOTIF_RATES!=0)) {
				if (PRINT_TO_SCREEN) cerr<<"recalculating conditionals...";
				for (a=0;a<Nalignments;a++) {
					if (PRINT_TO_SCREEN) cerr<< ".";
					for (p=ALI[a].leftend;p<(ALI[a].rightend);p++)
						if ((ALI[a].nextbad[p]>=(p+W))&&(ALI[a].nextgap[p]>=(p+W)))
							for (pos=0;pos<W;pos++) for (b=0;b<4;b++) {
								mcond[a][p][pos][b]=p_subtree_given_motif(&ALI[a],ALI[a].phyl.root,M[i+1].f[pos],M[i+1].K[pos],b,p+pos);
								if (BOTH_STRANDS) rccond[a][p][pos][b]=p_subtree_given_motif(&ALI[a],ALI[a].phyl.root,RC.f[pos],RC.K[pos],b,p+pos);
							}
				}
				if (PRINT_TO_SCREEN) cerr<<endl;
			}
			if (PRINT_TO_SCREEN) {
				cout << "log(LH)="<<loglh[i]<<endl;
				M[i+1].print();
				//return 0;
			}
		}
	}//end of iteration loop

i=bestiteration;
RC=M[i].make_rc();
M[i].rc=&RC;
if (PRINT_OUT) {
	out  << "pos\tA\tC\tG\tT\tK"<<endl;
	for (pos=0;pos<W;pos++) {
		out << pos <<"\t";
		for (b=0;b<4;b++) out <<M[i].f[pos][b]<<"\t";
		out<< M[i].K[pos]<< endl;
	}
	out << "Probable occurances of motif "<< n<<endl;
	out << "alignment\tpos\tposterior\tlog(LR)\tstrand\tseqs"<<endl;
}
if (PRINT_TO_SCREEN) cerr<<"...";
loglh[i]=0.0;
for (a=0;a<Nalignments;a++) {
	if (PRINT_TO_SCREEN) cerr<< ".";
	for (p=ALI[a].leftend;p<(ALI[a].rightend-W);p++)
		if ((ALI[a].nextbad[p]>=(p+W))&&(ALI[a].nextgap[p]>=(p+W))) {
			if ((EVOL_MODEL==1)||(UPDATE_MOTIF_RATES!=0)) {
				for (pos=0;pos<W;pos++) for (b=0;b<4;b++) {
					mcond[a][p][pos][b]=p_subtree_given_motif(&ALI[a],ALI[a].phyl.root,M[i].f[pos],M[i].K[pos],b,p+pos);
					if (BOTH_STRANDS) rccond[a][p][pos][b]=p_subtree_given_motif(&ALI[a],ALI[a].phyl.root,RC.f[pos],RC.K[pos],b,p+pos);
				}
			}
			double pbg=(1.0-M[i].pri)*lh_given_bg(p,p+W,bglh[a]);
			double pm=M[i].pri*lh_given_motif(&M[i],mcond[a][p]);
			double prc=0.0;
			if (BOTH_STRANDS) prc=M[i].pri*lh_given_motif(M[i].rc,rccond[a][p]);
			double Em= ((BOTH_STRANDS) ? 0.5*(pm+prc)/(0.5*(pm+prc)+pbg) : pm/(pm+pbg));
			if (Em>MIN_POST_PROB) {
				already_found[a][p]=1;
				if (PRINT_OUT) {
					out <<ALI[a].name<<"\t"<<p<<"\t"<<Em<<"\t"<<
					log(0.5*(lh_given_motif(&M[i],mcond[a][p])+lh_given_motif(M[i].rc,mcond[a][p]))) -
					log(lh_given_bg(p,p+W,bglh[a]))
					<<"\t"<<0.01*((int) (100.0*pm/(pm+prc)))<<"\t";
					//<<"\t"<<0.01*((int) 100.0*pm/(pm+prc))<<"\t";
					for (s=0;s<ALI[a].Nseqs;s++) {
						for (pos=0;pos<W;pos++) out <<ALI[a].seq[s].bases[p+pos]; out << " ";
					}
					out << endl;
				}
			}
			int r= (p>=OVERLAP) ? p-OVERLAP : 0;
			int l= ((p+OVERLAP)<(ALI[a].rightend-W)) ? p+OVERLAP : ALI[a].rightend-W;
			double temptot=0.0;
			for (pos=r;pos<l;pos++) if ((ALI[a].nextgap[pos]>=(pos+W))&&(ALI[a].nextbad[pos]>=(p+W))) {
				double temp=M[i].pri*lh_given_motif(&M[i],mcond[a][pos]);
				if (BOTH_STRANDS) temp=0.5*(temp+lh_given_motif(M[i].rc,rccond[a][pos]));
				temptot+=(temp/((1.0-M[i].pri)* lh_given_bg(pos,pos+W,bglh[a])+ temp));
			}
			if (temptot>1.0) Em /= temptot;
			if (BOTH_STRANDS) loglh[i]+=log(pbg + Em*(0.5*(pm+prc) + pbg));
			else loglh[i]+=log(pbg + Em*(pbg+pm));
			//if (BOTH_STRANDS) loglh[i]+=log(0.5*(pm+prc) + pbg);
			//else loglh[i]+=log(pm+pbg);
		}
}
if (PRINT_TO_SCREEN) cerr<<endl;
out << "Motif " << n <<" logLH= "<<loglh[i]<<" ("<<i<<")"<< endl;
if (GIVEN_MOTIF==2) n=N_MOTIFS;
}//end of motif number

}//end of main

double div_to_subs_JC (double R) { // Jukes-cantor parameter as a function of changed to id residues
	if ((0.0<R) && (R<3.0)) {
		double k=-0.75*log((3.0-R)/(3.0+3.0*R));
		return ((k>MIN_RATE) ? k : MIN_RATE);
	}
	else return -1.0;
}
double lh_pos(double * f, double * cond) {
	int b;
	double lh=0.0;
	for (b=0;b<4;b++) lh += f[b]*cond[b];
	return lh;
}

double lh_given_motif(struct motif * m, double ** cond) {
	int p,b;
	double lh=1.0; double temp;
	for (p=0;p<m->length;p++) {
		temp=0.0;
		//for (b=0;b<4;b++) cout << cond[p][b]<<"\t";
		//cout << p<<endl;
		for (b=0;b<4;b++) temp += ( cond[p][b] * m->f[p][b] );
		lh*=temp;
	}
	return lh;
}

double lh_given_bg(int start,int stop,double * lhpos) {
	int p;
	double lh=1.0;
	for (p=start;p<stop;p++) lh*=lhpos[p];
	if (lh==0) for (p=start;p<stop;p++) cout <<p<<" "<<lhpos[p]<<endl;
	return lh;
}


double p_subtree_given_base (struct alignment * ali, struct node * n, int base, int pos) { // recursion used to caluclate the likelihood of trees
	if (n->leaf) return ((ali->seq[n->number].asints[pos] == base) ? 1.0 : 0.0);
        int lb,rb;
        double Prl=0.0;
        double Prr=0.0;
        for (lb=0;lb<4;lb++) Prl+= ( p_subtree_given_base(ali,n->left,lb,pos)
					* ali->phyl.psubs[n->number][n->left->number][base][lb] );
        for (rb=0;rb<4;rb++) Prr+= ( p_subtree_given_base(ali,n->right,rb,pos)
					* ali->phyl.psubs[n->number][n->right->number][base][rb] );
        return Prr*Prl;
}

double p_subtree_given_motif (struct alignment * ali, struct node * n, double * f, double k, int base, int pos) {
	if (n->leaf) return ((ali->seq[n->number].asints[pos] == base) ? 1.0 : 0.0);
	int lb,rb;
        double Prl=0.0;
        double Prr=0.0;
	if (UPDATE_MOTIF_RATES!=3) {
	if (EVOL_MODEL==0) {
        	for (lb=0;lb<4;lb++) Prl+= ( p_subtree_given_motif(ali,n->left,f,k,lb,pos)
						* PJC(base,lb,k*n->lbr) );
		for (rb=0;rb<4;rb++) Prr+= ( p_subtree_given_motif(ali,n->right,f,k,rb,pos)
						* PJC(base,rb,k*n->rbr) );
	}
	else if (EVOL_MODEL==1) {
		for (lb=0;lb<4;lb++) Prl+= ( p_subtree_given_motif(ali,n->left,f,k,lb,pos)
						* PF81(base,lb,k*n->lbr,f) );
		for (rb=0;rb<4;rb++) Prr+= ( p_subtree_given_motif(ali,n->right,f,k,rb,pos)
						* PF81(base,rb,k*n->rbr,f) );

	}
	}
	else {
		for (lb=0;lb<4;lb++) Prl+= ( p_subtree_given_motif(ali,n->left,f,k,lb,pos)
						* PHB(base,lb,n->lbr,f,ali->bgfreq) );
		for (rb=0;rb<4;rb++) Prr+= ( p_subtree_given_motif(ali,n->right,f,k,rb,pos)
						* PHB(base,rb,n->rbr,f,ali->bgfreq) );
	}
	return Prr*Prl;
}

double PJC (int a, int b, double alpha) { //jukes-cantor probability of a to b in alpha//
	//cout << a<<"->"<<b<<endl;
	if (a!=b) return (0.25-0.25*exp((-4.0/3.0)*alpha));
	else return (0.25+0.75*exp((-4.0/3.0)*alpha));

}

double PF81 (int a, int b, double alpha, double * bgf) { //F81 probability of a to b in alpha
	if (a!=b) return bgf[b]*(1.0-exp(-alpha));
	else return (bgf[b]+(1.0-bgf[b])*exp(-alpha));
}

double PHB (int a, int b, double alpha, double * f, double * bgf) { //
	if (a!=b) {
		if (EVOL_MODEL==0) {
			if (fabs(f[a]-f[b])>0.0)
				return PJC(a,b,(alpha * log(f[b]/f[a]) / (1.0 - (f[a]/f[b]))));
			else return PJC(a,b,alpha);
		}
		else if (EVOL_MODEL==1) {
			double temp1=PF81(b,a,alpha,bgf)*f[b];
			double temp2=PF81(a,b,alpha,bgf)*f[a];
			if (fabs(f[a]-f[b])>0.0)
				return PF81(a,b,(alpha * log(temp1/temp2) / (1.0 - (temp2/temp1))),bgf);
			else return PF81(a,b,alpha,bgf);
		}
	}
	else {
		int base;
		double pr=1.0;
		for (base=0;base<4;base++) if (base != b)
			pr-=PHB(a,base,alpha,f,bgf);
		return pr;
	}
}


sequence::sequence() {
	length=0;
}

bool sequence::print(int start, int stop) {
	if (start>=stop) return 0;
	if (stop>=length) return 0;
	int p,b; char B[5]; B[0]='A'; B[1]='C'; B[2]='G'; B[3]='T'; B[4]='-';
	cout << name <<"("<<start<<"-"<<stop<<")\t";
	for (p=start;p<stop;p++) {
		if (asints[p]==-1) cout <<"N";
		else cout <<B[asints[p]];
	}
	cout << endl;
	return 1;
}
bool sequence::integerize() {
	length=bases.size();
	int p,b;
	//char B[4];
	asints = new int [length];
	//B[0]='A'; B[1]='C'; B[2]='G'; B[3]='T';
	//for (p=0;p<length;p++) for (b=0;b<4;b++) if (B[b] == bases[p]) asints[p]=b;
	//for (p=0;p<length;p++) cout <<seq[p]<<endl;
	for (p=0;p<length;p++) asints[p] = (
				(bases[p]=='A') ? 0 : (
					(bases[p]=='C') ? 1 : (
						(bases[p]=='G') ? 2 : (
							(bases[p]=='T') ? 3 : (
								(bases[p]=='-') ? 4 : -1
									)
								)
							)
						)
					);

	return 1;
}

node::node() {
	leaf=0;
	root=0;
	//seq_post = new double [4];
	//int b;
	//for (b=0;b<4;b++) seq_post[b]=0.0;
	base=4;
	lbr=abr=rbr=INITIAL_BRANCH;

}

bool node::print() {
	char B[4]; B[0]='A'; B[1]='C'; B[2]='G'; B[3]='T';
	int b;
	if (root) cout << "\troot"<<endl;
	else cout <<"\tn"<<anc->number<<" "<<endl;
	cout <<"\t|"<<endl;
	if (leaf) cout << "\tleaf" <<endl;
	else cout <<"n"<< left->number << "\t-\tn" <<right->number<<endl;
	//if (leaf){
	//	if (base == 4) cout <<"\tleaf"<<endl;
	//	else if (base==-1) cout << "\tgap" <<endl;
	//	else cout << "\t"<<B[base] <<endl;
	//}
	//else {
	//	//for (b=0;b<4;b++) cout << " "<<seq_post[b]; cout << endl;
	//	cout <<"n"<< left->number << "\t-\tn" <<right->number<<endl;
	//}
	return 1;
}

tree::tree() {
	root=0;
}


bool tree::print() {

	int n=0;
	for (n=0;n<Nnodes;n++) {
	//while (!done) {
		cout << "node " <<n<<endl;
		nodelist[n].print();
	}
	return 1;
}
bool tree::initialize_pairwise(double dis) {
	int n;
	Nnodes=3;
	nodelist=new struct node [3];
	nodelist[2].number=2;
	root= &nodelist[2];
	nodelist[2].root=1;
	nodelist[2].left = &nodelist[0];
	nodelist[2].right= &nodelist[1];
	nodelist[2].lbr=nodelist[2].rbr=0.5*dis;
	for (n=0;n<2;n++) {
		nodelist[n].number=n;
		nodelist[n].leaf=1;
		nodelist[n].anc= &nodelist[2];
		nodelist[n].abr=0.5*dis;
	}
	return 1;
}
bool tree::compute_average_F81_branches(struct alignment * ali, int na, int MAX_ITER, double err) {
if (Nnodes!=3) { cerr <<"works on pairwise trees only"<<endl; return 0; }
	ofstream his("tree.history");
	double loglh[MAX_ITER]; double prch,temp,temp2,templh,A,B;
	int p,i,b,a,stable=0;
	for (i=0;i<MAX_ITER;i++) {
		loglh[i]=prch=0.0;
		for (a=0;a<na;a++) for (p=ali[a].leftend;p<ali[a].rightend;p++) if ((ali[a].nextgap[p]!=p)&&(ali[a].nextbad[p]!=p)) {
			A=B=temp=temp2=templh=1.0;
			for (b=0;b<4;b++) {
				//felsenstein's formula - works at the root
				temp+= ali[a].bgfreq[b] * p_subtree_given_base(&ali[a],root->right,b,p);
				A+= ali[a].bgfreq[b] * p_subtree_given_base(&ali[a],root->left,b,p) *
					p_subtree_given_base(&ali[a],root->right,b,p);
				temp2+= ali[a].bgfreq[b] * p_subtree_given_base(&ali[a],root->left,b,p);
				templh+=ali[a].bgfreq[b] * p_subtree_given_base(&ali[a],root,b,p);
			}
			loglh[i]+=log(templh);
			B= temp * temp2;
			temp=1.0-exp(-root->lbr - root->rbr);
			prch += B * temp / (B*temp + A*(1.0-temp));
		}
		root->lbr=root->left->abr=root->right->abr=-0.5*log(1.0 - prch);
		if (i>0) {
			his << i <<"\t"<<loglh[i]<<"\t("<<(loglh[i-1]-loglh[i])<<")\t";
			his<<-log(1.0-prch)<<endl;
			if ((loglh[i-1]+err)>=loglh[i]) stable++;
			else stable=0;
			if (stable>=5) i=MAX_ITER;
		}
	}
	return 1;
}

bool tree::compute_F81_branches(struct alignment * ali, int MAX_ITER, double err) {
	if (Nnodes!=3) { cerr <<"works on pairwise trees only"<<endl; return 0; }
	ofstream his("tree.history");
	double loglh[MAX_ITER]; double prch,temp,temp2,templh,A,B;
	int p,i,b,a,stable=0;
	for (i=0;i<MAX_ITER;i++) {
		loglh[i]=prch=0.0;
		compute_PF81(ali->bgfreq);
		//for (a=0;a<4;a++) for (b=0;b<4;b++) cout << a <<" "<<b<<"->"<<psubs[2][0][a][b]<<endl;
		//if (i>1) return 0;
		for (p=ali->leftend;p<ali->rightend;p++) if ((ali->nextgap[p]!=p)&&(ali->nextbad[p]!=p)) {
			A=B=temp=temp2=templh=0.0;
			for (b=0;b<4;b++) {
				//felsenstein's formula - works at the root
				temp+= ali->bgfreq[b] * p_subtree_given_base(ali,root->right,b,p);
				A+= ali->bgfreq[b] * p_subtree_given_base(ali,root->left,b,p) *
					p_subtree_given_base(ali,root->right,b,p);
				temp2+= ali->bgfreq[b] * p_subtree_given_base(ali,root->left,b,p);
				templh+=ali->bgfreq[b] * p_subtree_given_base(ali,root,b,p);
			}
			//cout <<ali->name<<" "<< p << " "; for (b=0;b<4;b++) cout << " "<<ali->bgfreq[b]<< " x "<<p_subtree_given_base(ali,root,b,p);
			//cout<< " "<<templh<<endl;
			loglh[i]+=log(templh);
			B= temp * temp2;
			temp=1.0-exp(-root->lbr - root->rbr);
			prch += B * temp / (B*temp + A*(1.0-temp));
		}
		prch /= ((double) (ali->length - ali->gapped - ali->bad));
		//return 0;
		root->lbr=root->left->abr=root->right->abr=-0.5*log(1.0 - prch);
		//cout << i <<"new dis= "<<-0.5*log(1.0 - prch)<<" ("<<prch<<endl;
		if (i>0) {
			his << i <<"\t"<<loglh[i]<<"\t("<<(loglh[i-1]-loglh[i])<<")\t";
			his<<-log(1.0-prch)<<endl;
			if ((loglh[i-1]+err)>=loglh[i]) stable++;
			else stable=0;
			if (stable>=5) i=MAX_ITER;
		}
	}
	return 1;
}

bool tree::compute_PF81(double * bgf) {
	int a,b,n,m;
	psubs = new double *** [Nnodes];
	for (n=0;n<Nnodes;n++) psubs[n] = new double ** [Nnodes];
	for (m=0;m<Nnodes;m++) for (n=0;n<Nnodes;n++) psubs[n][m]= new double * [4];
	for (m=0;m<Nnodes;m++) for (n=0;n<Nnodes;n++) for (a=0;a<4;a++) psubs[n][m][a] = new double [4];
	for (n=0;n<Nnodes;n++) if (!nodelist[n].root) for (a=0;a<4;a++) for (b=0;b<4;b++) {
		psubs[nodelist[n].anc->number][n][a][b]=PF81(a,b,nodelist[n].abr,bgf);
	}
	return 1;
}
bool tree::compute_PJC() {
	int a,b,n,m;
	psubs = new double *** [Nnodes];
	for (n=0;n<Nnodes;n++) psubs[n] = new double ** [Nnodes];

	//for (a=0;a<4;a++) for (b=0;b<4;b++) psubs[a][b] = new double * [Nnodes];
	for (m=0;m<Nnodes;m++) for (n=0;n<Nnodes;n++) psubs[n][m]= new double * [4];
	for (m=0;m<Nnodes;m++) for (n=0;n<Nnodes;n++) for (a=0;a<4;a++) psubs[n][m][a] = new double [4];
	for (n=0;n<Nnodes;n++) if (!nodelist[n].root) for (a=0;a<4;a++) for (b=0;b<4;b++) {
		if (a != b) psubs[nodelist[n].anc->number][n][a][b] = 0.25 - 0.25*exp(-(4.0/3.0)*nodelist[n].abr);
		else psubs[nodelist[n].anc->number][n][a][b] = 0.25 + 0.75*exp(-(4.0/3.0)*nodelist[n].abr);
	}

	return 1;
}

bool tree::read_from_file(string treefile) {
	ifstream treedata(treefile.c_str());
	if (!treedata) return 0;
	char comm;
	int thisnum,num;
	treedata>>comm;
	if (comm == 't') {
		treedata>>Nnodes;
		//cout << Nnodes <<endl;
		nodelist=new struct node [Nnodes];
	}
	int i=0;
	while (i < Nnodes) {
		treedata>>comm; //print();
		//if (!comm) return 0;
		if ((comm == 'r')||(comm == 'n')) {
			treedata>> thisnum;
			//cout <<thisnum<<endl;
			//cout << i << endl;
			nodelist[thisnum].number=thisnum;
			if (comm=='r') {
				root = &nodelist[thisnum];
				nodelist[thisnum].root=1;
			}
			treedata>>comm;
			if (comm == 'l') {
				treedata>>num;
				nodelist[thisnum].left = &nodelist[num];
				nodelist[num].leaf=1;
				nodelist[num].number=num;
				nodelist[num].anc = &nodelist[thisnum];
				//nodelist[num].lbr=nodelist[num].rbr=-1.0;
				i++;
			}
			else if (comm == 'n') {
				treedata>>num;
				nodelist[thisnum].left = &nodelist[num];
				nodelist[num].anc= &nodelist[thisnum];
			}
			treedata>>comm;
			if (comm == 'l') {
				treedata>>num;
				nodelist[thisnum].right= &nodelist[num];
				nodelist[num].leaf=1;
				nodelist[num].number=num;
				nodelist[num].anc= &nodelist[thisnum];
				i++;
			}
			else if (comm == 'n') {
				treedata>>num;
				nodelist[thisnum].right= &nodelist[num];
				nodelist[num].anc= &nodelist[thisnum];
			}
			i++;
		}
		//else {cout << i ; print();}
	//print();
	}
	treedata>>comm;
	if (comm == 'b') {
		for (i=0;i<Nnodes;i++) if (!nodelist[i].leaf) {
			treedata>>num; if (num!=i) return 0;
			treedata>>nodelist[num].lbr;
			treedata>>nodelist[num].rbr;
			nodelist[num].left->abr=nodelist[num].lbr;
			nodelist[num].right->abr=nodelist[num].rbr;
		}
	}
	//cout << "read "<<endl;
	return 1;

}

alignment::alignment() {
	length=0;
	Nseqs=0;
	gapped=0;
	bad=0;
	seq = new struct sequence [MAX_ALIGN_SIZE];
	bgfreq = new double [4];
}
bool alignment::compute_nextgap() {
	char B[4]; B[0]='A'; B[1]='C'; B[2]='G'; B[3]='T';
	int p,b,s;
	int lastgap=length;
	int lastbad=length;
	leftend=0; rightend=-1;
	//double total=0.0;
	nextgap= new int [length];
	nextbad= new int [length];
	gapped=0;
	bad=0;
	for (b=0;b<4;b++) bgfreq[b]=0.0;
	for (p=length-1;p>=0;p--) {
		for (s=0;s<Nseqs;s++) {
			if ((seq[s].bases[p] != 'A') && (seq[s].bases[p] != 'C')
				&& (seq[s].bases[p] != 'G') && (seq[s].bases[p] != 'T')) {
				if (seq[s].bases[p] == '-') lastgap=p; //seq[s].asints[p]=-1;
				else lastbad=p;
			}
			else for (b=0;b<4;b++) if (seq[s].bases[p] == B[b]) //{
				bgfreq[b]+=1.0;
				//total+=1.0;
			//}

		}
		nextgap[p]=lastgap;
		nextbad[p]=lastbad;
		if (p==lastgap) gapped ++;
		else leftend=p;
		if (p==lastbad) bad ++;
		if (rightend==-1) if (gapped != (length-p)) rightend=p;
	}
	for (b=0;b<4;b++) bgfreq[b] /= ((double) (Nseqs*(length-gapped-bad)));
	//for (p=0;p<length;p++) cout << p <<" "<< nextgap[p] <<" "<<seq[p]<<endl;
	//cout << leftend<<" "<<rightend<<" "<<length<<" "<<endl;
	return 1;
}

float alignment::div() {
	float id=0.0;
	//float ungapped=0.0;
	int p,s1,s2;
	for (p=0;p<length;p++) {
		int thisposdiff=0;
		bool count=1;
		for (s1=0;s1<Nseqs;s1++) if ((seq[s1].asints[p]==4)||(seq[s1].asints[p]==-1)) count=0;
		if (count) {
			//for (s1=0;s1<Nseqs;s1++)
			for (s2=1;s2<Nseqs;s2++)
				if (seq[0].asints[p] != seq[s2].asints[p]) thisposdiff=1;
			//ungapped+=1.0;
			if (!thisposdiff) id+=1.0;
		}
	}
	id /= ((float) (length-gapped-bad));
	return (1.0-id);
}

bool alignment::read_tree(string filename, double * bgf) {
	int p,n,b;
	if (PRINT_TO_SCREEN) cerr << "reading tree from " <<filename<<"...";
	//phyl= new struct tree [length];
	if (!phyl.read_from_file(filename)) return 0;
	treelength=0.0;
	for (n=0;n<phyl.Nnodes;n++) if (!phyl.nodelist[n].root)
		treelength += phyl.nodelist[n].abr;
	//cout << global_tree.root->lbr<<endl;
	//double eqf[4]; eqf[0]=eqf[1]=eqf[2]=eqf[3]=0.25;
	//double loglh=0.0;
	//for (p=0;p<length;p++) if (nextgap[p]!=p) {
	//for (p=0;p<1;p++) {
		//if (!phyl[p].read_from_file(filename)) return 0;
		//phyl[p].print();
		//ALI[0].phyl[p].print();
		//for (n=0;n<phyl[p].Nnodes;n++) {
		//	if (phyl[p].nodelist[n].leaf) {
		//		phyl[p].nodelist[n].base=seq[n].asints[p];
				//for (b=0;b<4;b++) ALI[0].phyl[p].nodelist[n].seq_post[b]=0.0;
 		//		phyl[p].nodelist[n].seq_post[phyl[p].nodelist[n].base]=1.0;
		//	}
		//	else if ((global_tree.root->lbr==INITIAL_BRANCH)&&(global_tree.root->rbr==INITIAL_BRANCH)) {
		//			for (b=0;b<4;b++) phyl[p].nodelist[n].seq_post[b]=bgf[b];
		//	}
		//	else if (EVOL_MODEL==1) compute_post(phyl[p].nodelist + n, bgf);
		//	else if (EVOL_MODEL==0) compute_post_JC(phyl[p].nodelist + n, bgf);
		//}
	//	loglh+=log(compute_post_JC(phyl[p].root,bgf));
		//compute_post(phyl[p].root->left);
		//compute_post(phyl[p].root->right);
		//loglh+=compute_post(phyl[p].root, bgf);
	//}
	//cout << "initial bg log liklihood " << loglh <<endl;
	//if (EVOL_MODEL==0) {
	//if ((global_tree.root->lbr==INITIAL_BRANCH)&&(global_tree.root->rbr==INITIAL_BRANCH))
	//	optimize_branch_lengths(EPSILON,MAX_ITER);
	if ((phyl.root->lbr!=INITIAL_BRANCH)&&(phyl.root->rbr!=INITIAL_BRANCH))
		if (PRINT_TO_SCREEN) cerr << "branch lengths found" <<endl;
	else if (PRINT_TO_SCREEN) cerr << "branch lengths not found" <<endl;
	//}
	//loglh=0.0;
	//for (p=0;p<length;p++) if (nextgap[p]!=p)
	//	loglh+=log(compute_post_JC(phyl[p].root,bgf));
	//cout << "maximum bg liklihood "<<loglh<<endl;
	return 1;
}

bool alignment::read_fasta(string filename) {
	ifstream data(filename.c_str());
	if (!data) return 0;
	else if (PRINT_TO_SCREEN) cerr << "reading "<<filename<<"...";
	//string seqdata;
	Nseqs=-1;
	bool foundmax=0;
	int p; bool ext=1;
	for (p=filename.size();p>=0;p--) { //get rid of the last extension
		if (ext) { if (filename[p]=='.') ext=0; }
		else name=filename[p] + name;
		//cout << filename[p] <<" "<<ext<<" "<<name<<endl;
	}
	if (ext) name=filename; // if there was no extension
	//name[filename.size()]='\0';
	//name=filename;
	while ((!data.fail())&&(!foundmax)) {
		string line;
		getline(data,line);
		int linelength=line.size();
		//cout << linelength << endl;
		if ( line[0] == '>' ) {
			Nseqs++;
			if (Nseqs<MAX_ALIGN_SIZE) { for (p=1;p<linelength;p++) seq[Nseqs].name += line[p]; }
			else foundmax=1;
		}
		else if ((linelength>0)&&(!foundmax)) {
			seq[Nseqs].bases += line;
			//cout << line << endl;
			//for (p=0;p<linelength;p++) if (line[p] == '\n') cout << p <<endl;
		}
	}
	int s;
	if (Nseqs<MAX_ALIGN_SIZE) Nseqs++;
	for (s=0;s<Nseqs;s++) {
		seq[s].integerize();
		if (length < seq[s].length) length=seq[s].length;
	}
	compute_nextgap();
	//P_BG_COMPUTED=0;
	//pbg=new double [length];

	if (Nseqs==2) { // initialise the pairwise tree
		//if (EVOL_MODEL==0) {
		//double d=div();
		//double r=(1.0-d)/d;
                //treelength=-0.75*log((1.0-(r/3.0))/(1.0-r));
                //phyl.initialize_pairwise(treelength);

		treelength=div_to_subs_JC(1.0/(1.0-div()) - 1.0); //JC distance
		phyl.initialize_pairwise(treelength);
	}

	if (PRINT_TO_SCREEN) {
		if (EVOL_MODEL==0) cerr << "k="<<treelength<<" ("<<1.0-div()<<"%id)" <<endl;
		else cerr << "done"<<endl;
	}
	if (Nseqs>1) return 1;
	else return 0;
}
motif::motif() {
	//name = new char [MAX_NAME_LENGTH];
	//f = new double * [4];
	pri=MOTIF_PRIOR;
}
double motif::Pr(int s[],int start) {
	int pos; double p=1.0;
	for (pos=0;pos<length;pos++) p *= f[pos][s[start+pos]];
	return p;
}

struct motif motif::make_rc() {
	struct motif rc;
	int b, pos;
	int revcom[4]; revcom[0]=3; revcom[1]=2; revcom[2]=1; revcom[3]=0;
	rc.length=length;
	rc.pri=pri;
	rc.K = new double [length];
	rc.f = new double * [length];
	for (pos=0;pos<length;pos++) rc.f[pos] = new double [5];
	for (pos=0;pos<length;pos++) {
		for (b=0;b<4;b++) rc.f[pos][b]=f[length-pos-1][revcom[b]];
		rc.K[pos]=K[length-pos-1];
		rc.f[pos][4]=f[length-pos-1][4];
	}
	return rc;
}
bool motif::initialize_neutral_motif(int w, double pseudocounts) {
	int b, pos;
	length=w;
	//cout << "initialising matrix of size "<< length <<endl;
	f = new double * [length];
	K = new double [length];
	for (pos=0;pos<length;pos++) f[pos] = new double [5];
	for (pos=0;pos<length;pos++) {
		for (b=0;b<4;b++) f[pos][b]=0.25*pseudocounts;
		K[pos]=1.0;
		f[pos][4]=0.0;
	}
	return 1;
}


bool motif::print() {
	char B[4]; B[0]='A'; B[1]='C'; B[2]='G'; B[3]='T';
	int b,pos;
	cout << "pos\t";
	for (b=0;b<4;b++) cout << B[b] <<"\t";
	cout << "Kp"<<"\t"<<"Khb"<<endl;
	for (pos=0;pos<length;pos++) {
		cout << pos << "\t";
		for (b=0;b<4;b++) cout << 0.01*((int) (100.0*f[pos][b])) <<"\t";
		cout << K[pos]<< "\t"<<halpern_bruno(pos)<<endl;

	}
	return 1;
}
double motif::halpern_bruno(int pos) {
	double hbrate=0.0;
	int b1,b2;
	for (b1=0;b1<4;b1++) for (b2=0;b2<4;b2++) if (b2!=b1) {
		if (f[pos][b1]==f[pos][b2]) hbrate+=f[pos][b1];
		else hbrate+=(f[pos][b1]*log(f[pos][b2]/f[pos][b1])/(1-(f[pos][b1]/f[pos][b2])));
	}
	return (hbrate/3.0);
}
bool motif::read_from_file(string filename) {
	ifstream data(filename.c_str());
	if (!data) { cout << filename<<" not found" << endl; return 0; }
	length=-1;
	while (!data.fail()) {
		string line;
		getline(data,line);
		//cout << line << endl;
		length++;
	}
	cout << "motif of length " << length << " found" << endl;
	int b,pos;
	f = new double * [length];
	for (pos=0;pos<length;pos++) f[pos] = new double [5];
	//int pos=0;
	ifstream datatoo(filename.c_str());
	for (pos=0;pos<length;pos++) {
		for (b=0;b<4;b++) datatoo>>f[pos][b];
		f[pos][4]=0.0;
	}

	K=new double [length];
	for (pos=0;pos<length;pos++) K[pos]=1.0;
	name=filename;
	return 1;
}
